; CODE FOR PICAXE 18X BASED ALTIMETER DATALOGGER
; THIS IS THE TRANSMITTER SIDE
;
; BY JON SOWMAN - 2008
;
; ------ NOTES ------
; -------------------
; THE POWER PIN IS USED WITH THE SHUTDOWN PIN ON A MAX883
; HOLD IT HIGH FOR NORMAL OPERATION, SEND LOW TO SHUT DOWN
;
; TWO STATUS LEDS ARE REQUIRED AND ONE LINE FOR SERIAL OUT
; TO AN RF TRANSMITTER
;
; PINS CAN BE CHANGED USING THE SYMBOLS BELOW
;
; POWER SUPPLY SHOULD BE ABOVE 5.5V
;
; ---- STATUS LED CODES ----
; --------------------------
; GREEN + RED = WAITING FOR USER INPUT
; FLASHING GREEN AND RED = INITIALISING
; SOLID RED, NO GREEN = TEST CONNECTION MODE - LOOK FOR SOLID RED ON RECIEVER
; GREEN FLASH = RESETTING ZERO AND PEAK VALUES
; FLASHING GREEN, NO RED = SENDING DATA, EVERYTHING OK
; FLASHING GREEN, SOLID RED = INTERNAL EEPROM FULL BUT STILL LOGGING
; 3 GREEN FLASHES = SHUTTING DOWN
; -----------------------------------
; PROGRAM BEGINS HERE

#picaxe18x                     ' set ic


; THIS IS JUST DEFAULT DATA FOR TESTING
; IT CAN BE REMOVED IF YOU LIKE
EEPROM 0,(830)   ' zero
	EEPROM 2,(255)   ' peak
	EEPROM 3,(255)
EEPROM 4,(555)
EEPROM 6,(555)
EEPROM 8,(555)
EEPROM 10,(555)
EEPROM 12,(0)
EEPROM 13,(0)
EEPROM 14,(0)


' SYMBOLS THEN CHAPS? RIGHT OH

' -----------------
symbol statusled = 0           ' status led pin (green)
symbol statusled2 = 1          ' status led 2 (orange)
symbol adcpin = 0              ' MPX in
symbol serialoutpin = 3        ' serial out pin
symbol power = 7               ' regulator SHUTDOWN pin - DIODE PROTECT THIS! - low to kill
' -----------------
symbol startupdelay = 35       ' sensing circuit power up delay in ms !!!!!!!!!!!!!!!!!! DROP THIS
symbol dataoutdelay = 5        ' pause in ms between data outs
' -----------------
symbol adcvalue = w0           ' read MPX value into
symbol zerovalue = w1
' -----------------
symbol counts = 550            ' total number of readings  1000 ~= 1 min DEPENDING ON DELAYS
symbol internalfreq = 10       ' log internal one in X readings
' -----------------
symbol button1 = input1        ' start/stop log
symbol modejumper = input6     ' mode jumper
symbol button3 = input7        ' read internal over RF
' -----------------
symbol testbyte = $55
symbol eocbyte = "E"    ' end of conversation byte
symbol socbyte = "L"    ' start of conversation byte
symbol intbyte = "R"    ' transmit internal byte
' -----------------
symbol baud = N1200
' -----------------


' ARITE LUV, LETS INITIATE


init:
high power           ' TAKE CONTROL OF THE REGULATOR
low serialoutpin     ' SEND THE TX LINE LOW TO ELIMINATE NOISE

for b2 = 1 to 3
	high statusled
	high statusled2
	pause 100
	low statusled
	low statusled2
	pause 100
next b2
let b2 = 0    ' RESET THE VAR USED IN THE COUNTER

high statusled         ' TURN ON THE LEDS TO SHOW IN 'MAIN'
high statusled2


' VE MUST GET ZE BALL ROLLING


main:
if button1 = 1 then     ' BUTTON 1 = LOG
	low statusled
	low statusled2
	pause 500   ' debounce
	gosub thisiszero
	gosub testconn    ' SEND T TO CHECK CONNECTION
	pause 20
	serout serialoutpin, baud, (socbyte) ' SEND SOC BYTE
	pause 500    ' debounce
	let w2 = 0
	let w4 = 4   ' start internal log at position 4
	goto logdata            ' AND GO LOGGING
endif
if button3 = 1 then     ' BUTTON 3 = READ INTERNAL EEPROM OUT OVER SERIAL
	low statusled
	low statusled2
	pause 500   ' debounce
	serout serialoutpin, baud, (intbyte)  ' SEND READ COMMAND
	pause 50
	let b11 = 0
	gosub peekdata
endif
if modejumper = 1 then
	low statusled
	low statusled2
	pause 500       'debounce
	let w0 = 0
	goto automode
endif
goto main



automode:                   ' AUTOMATIC STARTUP MODE. SET BY THE JUMPER ON THE BOARD
	gosub thisiszero
	for w1 = 1 to 1500     ' 1000 = 20 secs
		high statusled2
		serout serialoutpin, baud, (testbyte) ' SEND TEST BYTE
		pause 20
		low statusled2
	next w1
	serout serialoutpin, baud, (socbyte) ' SEND SOC BYTE
	pause 500    ' wait for reciever to get ready
	let w2 = 0
	let w4 = 4   ' start internal log at position 4
goto logdata            ' AND GO LOGGING



preamble:
serout serialoutpin, baud, ($AA, $AA, $AA, $AA, $AA)
return




logdata:
	for w6 = 0 to counts
		if button1 = 1 then endlogging      ' check if STOP LOGGING button pressed
		gosub sensoron
		readadc10 adcpin, adcvalue
			' sertxd(#adcvalue,CR,LF) ;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		gosub sensoroff
		gosub checkpeak       ' check if this alt is higher than last reading
		gosub serialoutdata   ' send out data
		b11 = w6 // internalfreq
		'sertxd(#b11)
		if b11 = 0 then
			gosub loginternal
		endif
		pause dataoutdelay    ' pause for reciever to process
	next w6
goto endlogging




loginternal:
	if w4 <= 252 then       ' can write a word to 252 with 2 spaces left for ending 0s
		write w4, WORD w0
		pause 20
		let w4 = w4 + 2
	else
		high statusled2
		return
	endif
return



testconn:
	high statusled2
	serout serialoutpin, baud, (testbyte) ' SEND TEST BYTE
	pause 20
	if button1 = 0 then testconn     ' WAIT UNTIL BUTTON IS PRESSED TO START LOGGING
	low statusled2
return



endlogging:
low statusled2
pause 50
	serout serialoutpin, baud, (eocbyte,eocbyte)
	pause 50
	serout serialoutpin, baud, (eocbyte,eocbyte)
	pause 50
	serout serialoutpin, baud, (eocbyte,eocbyte)
	pause 50
	serout serialoutpin, baud, (eocbyte,eocbyte)
	pause 50
	serout serialoutpin, baud, (eocbyte,eocbyte)
	pause 50
write w4, 0,0   ' pair of 0s to denote the enddd of internal data
pause 20
goto shutdown



checkpeak:                 ' if this is new peak, log to internal eeprom
	read 2, WORD w1
	'sertxd(#adcvalue," - ",#w1,CR,LF)
	if w1 >= adcvalue then
		write 2, b0,b1
		pause 10 '!!!!! THIS WAS CHANGED FROM 20
	endif
return




sensoron:          ' turn on the sensing circuit
	high statusled
	pause startupdelay
return




sensoroff:         ' turn off the sensing circuit
	low statusled
return




serialoutdata:
	'sertxd(#w0)
	serout serialoutpin, baud, (b0,b1)
return






peekdata:
	high statusled2
	read b11, b12,b13
	sertxd(#w6,CR,LF) '!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if w6 = 0 then
		'gosub delimit
		serout serialoutpin, baud, (eocbyte,eocbyte)
		goto shutdown
	endif
	serout serialoutpin, baud, (b12,b13)  'makes up w6
	low statusled2
	'sertxd (#w6," | ")
	pause 30
	'gosub delimit
	let b11 = b11 + 2
goto peekdata





thisiszero:                    ' put the current reading in w1
	gosub sensoron
	readadc10 adcpin, zerovalue
	gosub sensoroff
	write 0, WORD w1   ' write to first internal eeprom position
	pause 20
	write 2, 255,255   ' reset the peak value
	pause 20
	high statusled
	pause 300
	low statusled
return







shutdown:       ' flash a few times and shutdown
low statusled2
high statusled
pause 150
low statusled
pause 150
high statusled
pause 150
low statusled
pause 150
high statusled
pause 150
low statusled
pause 150
high statusled
pause 550
low statusled
low power       ' power down the system
pause 1000
end

; ----- END PROGRAM -----











